/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.system.repository.artifact;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.kernel.api.server.PetalsException;

/**
 * A local artifact repository implementation. Folders strucutre is :<br>
 * $PETALS/artifacts/<br>
 * &nbsp;&nbsp;- $PETALS/artifacts/components<br>
 * &nbsp;&nbsp;- $PETALS/artifacts/sa<br>
 * &nbsp;&nbsp;- $PETALS/artifacts/sl<br>
 * 
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class LocalArtifactRepositoryService implements ArtifactRepositoryService {

    private static final String COMPONENTS = "components";

    private static final String SA = "sa";

    private static final String SL = "sl";

    private File rootPath;

    private File componentPath;

    private File slPath;

    private File saPath;

    public LocalArtifactRepositoryService(final File rootPath) {

        this.rootPath = rootPath;
        if (!rootPath.exists()) {
            rootPath.mkdirs();
        }

        componentPath = new File(rootPath, COMPONENTS);
        if (!componentPath.exists()) {
            componentPath.mkdirs();
        }

        slPath = new File(rootPath, SL);
        if (!slPath.exists()) {
            slPath.mkdirs();
        }

        saPath = new File(rootPath, SA);
        if (!saPath.exists()) {
            saPath.mkdirs();
        }
    }

    public List<Artifact> getComponents() {
        Jbi descriptor = null;
        List<Artifact> result = null;
        List<File> list = new ArrayList<File>();
        File[] components = componentPath.listFiles();
        for (File component : components) {
            descriptor = readDescriptor(component);
            if (descriptor != null && descriptor.getComponent() != null) {
                list.add(component);
            }
        }

        result = new ArrayList<Artifact>(list.size());
        // create the artifact information from the file...
        for (File file : list) {
            // create the artifact
            Artifact artifact = new Artifact();
            artifact.setFile(file);
            artifact.setName(getComponentName(file));
            // artifact.setVersion(version);
            // artifact.setGroupId(groupId)
            result.add(artifact);
        }

        return result;
    }

    public List<Artifact> getServiceAssemblies() {
        Jbi descriptor = null;
        List<Artifact> result = null;
        List<File> list = new ArrayList<File>();
        File[] sas = saPath.listFiles();
        for (File sa : sas) {
            descriptor = readDescriptor(sa);
            if (descriptor != null && descriptor.getServiceAssembly() != null) {
                list.add(sa);
            }
        }

        result = new ArrayList<Artifact>(list.size());
        // create the artifact information from the file...
        for (File file : list) {
            // create the artifact
            Artifact artifact = new Artifact();
            artifact.setFile(file);
            artifact.setName(getServiceAssemblyName(file));
            // artifact.setVersion(version);
            // artifact.setGroupId(groupId)
            result.add(artifact);
        }

        return result;
    }

    public List<String> getComponentsName() {
        List<Artifact> artList = getComponents();
        List<String> result = new ArrayList<String>(artList.size());
        for (Artifact artifact : artList) {
            result.add(artifact.getName());
        }
        return result;
    }

    public File getComponentsPath() {
        return componentPath;
    }

    public File getSaPath() {
        return saPath;
    }

    public File getRootPath() {
        return this.rootPath;
    }

    public boolean addArtifact(Artifact artifact) throws PetalsException {
        boolean result = true;
        if (artifact != null && artifact.getFile() != null && artifact.getFile().exists()) {
            File file = artifact.getFile();
            Jbi desc = readDescriptor(file);
            try {
                File out = null;
                if (desc != null && desc.getComponent() != null) {
                    out = new File(componentPath, desc.getComponent().getIdentification()
                            .getName()
                            + ".zip");
                    FileUtils.copyFile(file, out);
                } else if (desc != null && desc.getServiceAssembly() != null) {
                    out = new File(saPath, desc.getServiceAssembly().getIdentification()
                            .getName()
                            + ".zip");
                    FileUtils.copyFile(file, out);
                } else if (desc != null && desc.getSharedLibrary() != null) {
                    out = new File(saPath, desc.getSharedLibrary().getIdentification()
                            .getName()
                            + ".zip");
                    FileUtils.copyFile(file, out);
                } else {
                    result = false;
                }
            } catch (IOException e) {
                throw new PetalsException(e.getMessage());
            }
        }
        return result;
    }

    /**
     * Read the descriptor from the zip file... TODO : Move in utils
     * 
     * @param file
     * @return
     */
    private Jbi readDescriptor(File zfile) {
        Jbi result = null;
        try {

            final ZipFile zipFile = new ZipFile(zfile);
            final ZipEntry jbiDescriptorZipEntry = zipFile.getEntry("META-INF/jbi.xml");
            final InputStream jbiDescriptorInputStream = zipFile
                    .getInputStream(jbiDescriptorZipEntry);

            // Load the JBI descriptor
            result = JBIDescriptorBuilder.buildJavaJBIDescriptor(jbiDescriptorInputStream);

        } catch (JBIDescriptorException e) {
        } catch (IOException e) {
        }

        return result;
    }

    /**
     * Get the component name from the artifact.
     * 
     * @param file
     * @return
     */
    protected String getComponentName(File file) {
        String result = null;
        Jbi desc = readDescriptor(file);
        if (desc != null) {
            org.ow2.petals.jbi.descriptor.original.generated.Component component = desc
                    .getComponent();
            if (component != null) {
                // should always have a name...
                result = component.getIdentification().getName();
            }
        }
        return result;
    }

    /**
     * 
     * @param file
     * @return
     */
    protected String getSharedLibraryName(File file) {
        String result = null;
        Jbi desc = readDescriptor(file);
        if (desc != null) {
            Jbi.SharedLibrary sl = desc.getSharedLibrary();
            if (sl != null) {
                // should always have a name...
                result = sl.getIdentification().getName();
            }
        }
        return result;
    }

    protected String getServiceAssemblyName(File file) {
        String result = null;
        Jbi desc = readDescriptor(file);
        if (desc != null) {
            org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly sa = desc
                    .getServiceAssembly();
            if (sa != null) {
                // should always have a name...
                result = sa.getIdentification().getName();
            }
        }
        return result;
    }

    public List<String> getServiceAssembliesName() {
        List<Artifact> artList = getServiceAssemblies();
        List<String> result = new ArrayList<String>(artList.size());
        for (Artifact artifact : artList) {
            result.add(artifact.getName());
        }
        return result;
    }

    public File getServiceAssemblyPath() {
        return saPath;
    }

    public List<String> getSharedLibrariesName() {
        List<Artifact> artList = getSharedLibraries();
        List<String> result = new ArrayList<String>(artList.size());
        for (Artifact artifact : artList) {
            result.add(artifact.getName());
        }
        return result;
    }

    public File getSharedLibraryPath() {
        return slPath;
    }

    public List<Artifact> getSharedLibraries() {
        Jbi descriptor = null;
        List<Artifact> result = null;
        List<File> list = new ArrayList<File>();
        File[] sls = slPath.listFiles();
        for (File sl : sls) {
            descriptor = readDescriptor(sl);
            if (descriptor != null && descriptor.getSharedLibrary() != null) {
                list.add(sl);
            }
        }

        result = new ArrayList<Artifact>(list.size());
        // create the artifact information from the file...
        for (File file : list) {
            // create the artifact
            Artifact artifact = new Artifact();
            artifact.setFile(file);
            artifact.setName(getSharedLibraryName(file));
            // artifact.setVersion(version);
            // artifact.setGroupId(groupId)
            result.add(artifact);
        }

        return result;
    }

}
